/*
 * Copyright (C) 2009 RDC Semiconductor Co.,Ltd
 * All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sub license, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT.
 * IN NO EVENT SHALL PRECISION INSIGHT AND/OR ITS SUPPLIERS BE LIABLE FOR
 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * For technical support : 
 *     <rdc_xorg@rdc.com.tw>
 */
 #ifndef __TV_H__
#define __TV_H__


#define NTSC                 0
#define PAL                  1

#define CVBS                 1
#define SVideo               2

#define TVVESAModeCnt        8 
#define TVSupportModeCnt     5 


#define Reg_FS473HPos               0x0
#define Reg_FS473VPos               0x2
#define Reg_FS473HScaler            0xA
#define Reg_FS473HLineBuffer        0x4

struct _TVTBLDATA
{
    unsigned char TVModeIndex;
    unsigned short TVNTSCTP;
    unsigned short TVPALTP;
}__attribute__((packed));; 

struct _TVVESAMODETBL
{
    unsigned short  TVMode_ID_8bpp;
    unsigned short  TVMode_ID_16bpp;
    unsigned short  TVMode_ID_32bpp;
    unsigned char   TVVESAModeIndex;
}__attribute__((packed));; 

struct _FSVSRegdata
{
    unsigned short  VTotal;
    unsigned char   ucData[20];
}__attribute__((packed));;

typedef struct _FSVSRegdata FSVSRegdata;

struct _FSVScaler
{
    unsigned char ucTVType;
    unsigned short wHRes;
    unsigned short wVRes;
    FSVSRegdata *ScalerTable;
}__attribute__((packed));;


typedef struct _TVTBLDATA TVTBLDATA;
typedef struct _TVVESAMODETBL TVVESAMODETBL;
typedef struct _FSVScaler FSVScaler;

typedef struct _TV_Disp_Info
{
    unsigned char    ucCCRSLevel;        
    unsigned char    ucHPosition;        
    unsigned char    ucVPosition;        
    int              iHScaler;           
    unsigned char    ucVScaler;          
} TV_Disp_Info,*PTV_Disp_Info;



typedef struct _TVENCODERINFO
{
    unsigned char    bTV_HDTV;           
    unsigned char    bSupportHDTV;       
    unsigned char    EnCoder;            
    unsigned char    I2CPort;            
    unsigned char    SlaveAddr;          
    unsigned char    DisplayPath;        
    unsigned char    TVType;             
    unsigned char    TVConnect;          
    unsigned char    HDTVType;           
    unsigned char    HDTVConnect;
    unsigned char    CCRSLevel;          
    unsigned char    ucHPosition;        
    unsigned char    ucVPosition;        
    int              iHScaler;           
    unsigned char    ucVScaler;          
    unsigned short   TVOut_HSize;        
    unsigned short   TVOut_VSize;        
    unsigned char    bEnableHPanning;     
    unsigned char    bEnableVPanning;    
    unsigned short   wPreX;              
    unsigned short   wPreY;              
    unsigned short   wPanningViewX;      
    unsigned short   wPanningViewY;      
    unsigned long    dwPanningDispAddr;
    unsigned short   ModeHSize;          
    unsigned short   ModeVSize;          
    unsigned char    bTVModeSupport;     
} TVENCODERINFO, *PTVENCODERINFO;


static const FSVSRegdata FSVSNTSC640x480[5]= {
   
        {565, 0x19, 0x00, 0x58, 0x44, 0x8A, 0x00, 0x8A, 0x00, 0x8A, 0x00, 0x50, 0x23, 0x00, 0x00, 0x31, 0x23, 0x00, 0x00, 0xd0, 0xed},
        {560, 0x16, 0x00, 0xb8, 0x22, 0x87, 0x00, 0x86, 0x00, 0x87, 0x00, 0x00, 0x28, 0x00, 0x00, 0x31, 0x23, 0x00, 0x00, 0x00, 0xf0},
        {555, 0x14, 0x00, 0x18, 0x33, 0x89, 0x00, 0x88, 0x00, 0x89, 0x00, 0x9b, 0x5d, 0x00, 0x00, 0x78, 0x55, 0x00, 0x00, 0x29, 0xf2},
        {550, 0x17, 0x00, 0x28, 0x23, 0x88, 0x00, 0x87, 0x00, 0x88, 0x00, 0x64, 0x00, 0x00, 0x00, 0x5b, 0x00, 0x00, 0x00, 0x5d, 0xf4},
        {545, 0x18, 0x00, 0xc1, 0x21, 0x84, 0x00, 0x83, 0x00, 0x84, 0x00, 0x94, 0x2a, 0x00, 0x00, 0x31, 0x23, 0x00, 0x00, 0x9b, 0xf6}
};

static const FSVSRegdata FSVSNTSC800x600[5]= {
   
        {706, 0x1c, 0x00, 0x1d, 0x22, 0x82, 0x00, 0x82, 0x00, 0x82, 0x00, 0x20, 0x2c, 0x00, 0x00, 0xa9, 0x1a, 0x00, 0x00, 0x5e, 0xbe},
        {699, 0x1e, 0x00, 0xd4, 0x23, 0x84, 0x00, 0x84, 0x00, 0x84, 0x00, 0x90, 0x0e, 0x00, 0x00, 0xe3, 0x08, 0x00, 0x00, 0x46, 0xc0},
        {692, 0x1a, 0x00, 0x48, 0x23, 0x84, 0x00, 0x84, 0x00, 0x84, 0x00, 0x40, 0x2b, 0x00, 0x00, 0xa9, 0x1a, 0x00, 0x00, 0x38, 0xc2},
        {685, 0x19, 0x00, 0x78, 0x22, 0x83, 0x00, 0x83, 0x00, 0x83, 0x00, 0x90, 0x08, 0x00, 0x00, 0x55, 0x05, 0x00, 0x00, 0x34, 0xc4},
        {678, 0x25, 0x00, 0x94, 0x34, 0x84, 0x00, 0x84, 0x00, 0x84, 0x00, 0x20, 0x0e, 0x00, 0x00, 0xe3, 0x08, 0x00, 0x00, 0x3a, 0xc6}
};

static const FSVSRegdata FSVSNTSC1024x768[5]= {
   
        {904, 0x25, 0x00, 0x11, 0x23, 0x03, 0x00, 0x83, 0x00, 0x81, 0x00, 0x80, 0x38, 0x00, 0x00, 0xe5, 0x29, 0x00, 0x00, 0xac, 0x94},
        {897, 0x21, 0x00, 0xb5, 0x22, 0x03, 0x00, 0x83, 0x00, 0x81, 0x00, 0x70, 0x01, 0x00, 0x00, 0x13, 0x01, 0x00, 0x00, 0xd5, 0x95},
        {890, 0x1d, 0x00, 0x98, 0x23, 0x05, 0x00, 0x85, 0x00, 0x82, 0x00, 0x20, 0x0b, 0x00, 0x00, 0x61, 0x08, 0x00, 0x00, 0x02, 0x97},
        {883, 0x19, 0x00, 0x01, 0x22, 0x03, 0x00, 0x83, 0x00, 0x81, 0x00, 0x30, 0x37, 0x00, 0x00, 0xe5, 0x29, 0x00, 0x00, 0x35, 0x98},
        {876, 0x15, 0x00, 0xa9, 0x21, 0x03, 0x00, 0x83, 0x00, 0x81, 0x00, 0x40, 0x12, 0x00, 0x00, 0xf7, 0x0d, 0x00, 0x00, 0x6c, 0x99}
};

static const FSVSRegdata FSVSPAL640x480[5]= {
   
        {560, 0x1a, 0x00, 0x9d, 0x21, 0x84, 0x00, 0x04, 0x00, 0x84, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x01, 0x00, 0xb6, 0x1d},
        {555, 0x18, 0x00, 0xc0, 0x22, 0x88, 0x00, 0x08, 0x00, 0x88, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x01, 0x00, 0x49, 0x20},
        {550, 0x14, 0x00, 0x44, 0x22, 0x88, 0x00, 0x08, 0x00, 0x88, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x01, 0x00, 0xe8, 0x22},
        {545, 0x1a, 0x00, 0x91, 0x21, 0x84, 0x00, 0x04, 0x00, 0x84, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x01, 0x00, 0x93, 0x25},
        {540, 0x14, 0x00, 0x2d, 0x21, 0x84, 0x00, 0x04, 0x00, 0x84, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x01, 0x00, 0x4b, 0x28}
};

static const FSVSRegdata FSVSPAL800x600[5]= {
   
        {699, 0x1e, 0x00, 0x28, 0x45, 0x87, 0x00, 0x87, 0x00, 0x87, 0x00, 0x03, 0x0a, 0x00, 0x00, 0x53, 0x07, 0x00, 0x00, 0xe5, 0xe4},
        {692, 0x16, 0x00, 0xc0, 0x43, 0x87, 0x00, 0x87, 0x00, 0x87, 0x00, 0xbc, 0x1d, 0x00, 0x00, 0xf9, 0x15, 0x00, 0x00, 0x36, 0xe7},
        {685, 0x1a, 0x00, 0x40, 0x23, 0x85, 0x00, 0x85, 0x00, 0x85, 0x00, 0xe3, 0x05, 0x00, 0x00, 0x65, 0x04, 0x00, 0x00, 0x93, 0xe9},
        {678, 0x15, 0x00, 0x98, 0x22, 0x85, 0x00, 0x85, 0x00, 0x85, 0x00, 0xb6, 0x09, 0x00, 0x00, 0x53, 0x07, 0x00, 0x00, 0xfc, 0xeb},
        {671, 0x1a, 0x00, 0xbc, 0x23, 0x86, 0x00, 0x86, 0x00, 0x86, 0x00, 0xd5, 0x1c, 0x00, 0x00, 0xf9, 0x15, 0x00, 0x00, 0x73, 0xee}
};

static const FSVSRegdata FSVSPAL1024x768[5]= {
   
        {897, 0x20, 0x00, 0x50, 0x23, 0x05, 0x00, 0x85, 0x00, 0x82, 0x00, 0x2d, 0x08, 0x00, 0x00, 0x53, 0x07, 0x00, 0x00, 0x5f, 0xb2},
        {890, 0x1e, 0x00, 0x14, 0x23, 0x05, 0x00, 0x85, 0x00, 0x82, 0x00, 0xde, 0x04, 0x00, 0x00, 0x65, 0x04, 0x00, 0x00, 0xc6, 0xb3},
        {883, 0x1a, 0x00, 0xa4, 0x22, 0x05, 0x00, 0x85, 0x00, 0x82, 0x00, 0x25, 0x18, 0x00, 0x00, 0xf9, 0x15, 0x00, 0x00, 0x33, 0xb5},
        {876, 0x24, 0x00, 0xa4, 0x33, 0x05, 0x00, 0x85, 0x00, 0x82, 0x00, 0xfc, 0x07, 0x00, 0x00, 0x53, 0x07, 0x00, 0x00, 0xa5, 0xb6},
        {869, 0x16, 0x00, 0xf8, 0x22, 0x07, 0x00, 0x87, 0x00, 0x83, 0x00, 0xc3, 0x17, 0x00, 0x00, 0xf9, 0x15, 0x00, 0x00, 0x1e, 0xb8}
};

static const FSVSRegdata FSVSNTSC720x480[5]= {
   
        {565, 0x1c, 0x00, 0x4c, 0x23, 0x06, 0x00, 0x86, 0x00, 0x86, 0x00, 0x30, 0x15, 0x00, 0x00, 0x8d, 0x13, 0x00, 0x00, 0xe0, 0xed},
        {560, 0x14, 0x00, 0x08, 0x33, 0x08, 0x00, 0x88, 0x00, 0x88, 0x00, 0x00, 0x03, 0x00, 0x00, 0xcb, 0x02, 0x00, 0x00, 0x00, 0xf0},
        {555, 0x1e, 0x00, 0xed, 0x21, 0x03, 0x00, 0x83, 0x00, 0x83, 0x00, 0xd0, 0x14, 0x00, 0x00, 0x8d, 0x13, 0x00, 0x00, 0x29, 0xf2},
        {550, 0x19, 0x00, 0xdc, 0x22, 0x06, 0x00, 0x06, 0x00, 0x86, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x01, 0x00, 0x5d, 0xf4},
        {545, 0x15, 0x00, 0x4d, 0x21, 0x03, 0x00, 0x83, 0x00, 0x83, 0x00, 0x70, 0x14, 0x00, 0x00, 0x8d, 0x13, 0x00, 0x00, 0x9b, 0xf6}
};

static const FSVSRegdata FSVSPAL720x576[5]= {
   
        {672, 0x1e, 0x00, 0xfc, 0x22, 0x05, 0x00, 0x85, 0x00, 0x85, 0x00, 0xa0, 0x02, 0x00, 0x00, 0x71, 0x02, 0x00, 0x00, 0x18, 0xee},
        {666, 0x15, 0x00, 0x28, 0x33, 0x08, 0x00, 0x88, 0x00, 0x88, 0x00, 0x9a, 0x02, 0x00, 0x00, 0x71, 0x02, 0x00, 0x00, 0x3d, 0xf0},
        {660, 0x18, 0x00, 0xed, 0x21, 0x04, 0x00, 0x04, 0x00, 0x84, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x01, 0x00, 0x6c, 0xf2},
        {654, 0x17, 0x00, 0x64, 0x33, 0x08, 0x00, 0x88, 0x00, 0x88, 0x00, 0x8e, 0x02, 0x00, 0x00, 0x71, 0x02, 0x00, 0x00, 0xa5, 0xf4},
        {648, 0x1e, 0x00, 0x60, 0x23, 0x06, 0x00, 0x86, 0x00, 0x86, 0x00, 0x88, 0x02, 0x00, 0x00, 0x71, 0x02, 0x00, 0x00, 0xe9, 0xf6}
};

static const FSVScaler FocusVScaleringTable[TVVESAModeCnt]={
        {NTSC,  640, 480, FSVSNTSC640x480},
        {NTSC,  720, 480, FSVSNTSC720x480},
        {NTSC,  800, 600, FSVSNTSC800x600},
        {NTSC, 1024, 768, FSVSNTSC1024x768},
        {PAL ,  640, 480, FSVSPAL640x480},
        {PAL ,  720, 576, FSVSPAL720x576},
        {PAL ,  800, 600, FSVSPAL800x600},
        {PAL , 1024, 768, FSVSPAL1024x768}
};

#endif 

